# Copyright (c) 2013 Galah Group LLC
# Copyright (c) 2013 Other contributers as noted in the CONTRIBUTERS file
#
# This file is part of galah-interact-python.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Galah Interact is a framework for creating Test Harnesses that grade students'
programming assignments. For general project information, check out the project
page on GitHub: https://github.com/galah-group/galah-interact-python

Galah Interact was originally created by Galah Group LLC and is licensed under
the Apache license version 2.0. A copy of the license is available in the root
directory of the git repo in the file LICENSE, and online at
http://www.apache.org/licenses/LICENSE-2.0. Please ensure your use of this
library is within your rights per that license.

Other contributers have given their valuable time to this project in order to
make it better, and they are listed in the CONTRIBUTERS file in the root
directory of the git repo.

"""

from core import *

import pretty
import execute
import parse
import standardtests
